/*
   本ファイルは、Mersenne Twister（松本眞氏・西村拓士氏）の C言語のソースコードを Java で書き直したものである。
   以下は、そのライセンス条文（BSDライセンス）である。

	   A C-program for MT19937, with initialization improved 2002/1/26.
	   Coded by Takuji Nishimura and Makoto Matsumoto.

	   Before using, initialize the state by using init_genrand(seed)  
	   or init_by_array(init_key, key_length).

	   Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
	   All rights reserved.                          

	   Redistribution and use in source and binary forms, with or without
	   modification, are permitted provided that the following conditions
	   are met:

		 1. Redistributions of source code must retain the above copyright
			notice, this list of conditions and the following disclaimer.

		 2. Redistributions in binary form must reproduce the above copyright
			notice, this list of conditions and the following disclaimer in the
			documentation and/or other materials provided with the distribution.

		 3. The names of its contributors may not be used to endorse or promote 
			products derived from this software without specific prior written 
			permission.

	   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
	   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
	   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
	   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
	   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
	   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
	   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
	   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
	   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
	   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


	   Any feedback is very welcome.
	   http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
	   email: m-mat @ math.sci.hiroshima-u.ac.jp (remove space)
*/


package mersenne_twister;

public class MersenneTwister {
	/* Period parameters */  
	private final int N = 624;
	private final int M = 397;
	private final int MATRIX_A = 0x9908b0df;	/* 定数ベクトル a */
	private final int UPPER_MASK = 0x80000000;	/* 最上位ビット */
	private final int LOWER_MASK = 0x7fffffff;	/* それ以外のビット */

	private int mt[] = new int[N];	/* 状態ベクトルの配列 */
	private int mti = N + 1;	/* これが初期値 N + 1 の侭であれば、状態ベクトルは初期化されていないと言える */

	/* シード値による初期化 */
	private void init_genrand(int s){
		mt[0] = s & 0xffffffff;
		for (mti = 1; mti < N; mti++){
			mt[mti] = (1812433253 * (mt[mti - 1] ^ (mt[mti - 1] >>> 30)) + mti);
			/* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
			/* In the previous versions, MSBs of the seed affect   */
			/* only MSBs of the array mt[].                        */
			/* 2002/01/09 modified by Makoto Matsumoto             */
		}
	}
	public MersenneTwister(int s){ init_genrand(s); }

	/* 配列による初期化 */
	/* 引数 init_key は初期化に用いる配列である */
	/* 引数 key_length は初期化に用いる部分の長さである */
	/* 2004年2月26日に、C++ 用に変更が成された */
	public MersenneTwister(int init_key[], int key_length){
		int i = 1, j = 0, k;
		init_genrand(19650218);

		for (k = (N > key_length) ? N : key_length; k > 0; k--) {
			mt[i] = (mt[i] ^ ((mt[i - 1] ^ (mt[i - 1] >>> 30)) * 1664525)) + init_key[j] + j;	/* 非線形 */
			i++; j++;
			if (i >= N){
				mt[0] = mt[N-1];
				i = 1;
			}
			if (j >= key_length){
				j = 0;
			}
		}

		for (k = N - 1; k > 0; k--) {
			mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >>> 30)) * 1566083941)) - i;	/* 非線形 */
			i++;
			if (i >= N){
				mt[0] = mt[N-1];
				i = 1;
			}
		}

		mt[0] = 0x80000000;	/* 状態ベクトルの非零性を保証するため最上位ビットを1にする */ 
	}

	/* 32ビットの乱数を生成する */
	public long genrand_int32(){
		int y;
		int mag01[] = {0x0, MATRIX_A};

		if (mti >= N){	/* 1度にNワード生成する */
			int kk;

			if (mti == N + 1){	/* もし状態ベクトル初期化されていなければ */
				init_genrand(5489);	/* a default initial seed is used */
			}

			for (kk = 0; kk < N - M; kk++){
				y = (mt[kk] & UPPER_MASK) | (mt[kk + 1] & LOWER_MASK);
				mt[kk] = mt[kk + M] ^ (y >>> 1) ^ mag01[(int)(y & 0x1)];
			}
			for (; kk < N - 1; kk++){
				y = (mt[kk] & UPPER_MASK) | (mt[kk + 1] & LOWER_MASK);
				mt[kk] = mt[kk + M - N] ^ (y >>> 1) ^ mag01[(int)(y & 0x1)];
			}
			y = (mt[N - 1] & UPPER_MASK) | (mt[0] & LOWER_MASK);
			mt[N - 1] = mt[M - 1] ^ (y >>> 1) ^ mag01[(int)(y & 0x1)];

			mti = 0;
		}

		y = mt[mti++];

		/* Tempering */
		y ^= (y >>> 11);
		y ^= (y << 7) & 0x9d2c5680;
		y ^= (y << 15) & 0xefc60000;
		y ^= (y >>> 18);

		return y & 0xffffffffL;
	}

	/* 31ビットの乱数を生成する */
	public int genrand_int31(){
		return (int)(genrand_int32() >>> 1L);
	}

	/* 区間 [0, 1] の32ビット浮動小数点数の乱数を生成する */
	public double genrand_real1(){
		return genrand_int32() * (1.0 / 4294967295.0);	/* 除数は 2 ^ 32 - 1 */ 
	}

	/* 区間 [0, 1) の32ビット浮動小数点数の乱数を生成する */
	public double genrand_real2(){
		return genrand_int32() * (1.0 / 4294967296.0);	/* 除数は 2 ^ 32 */ 
	}

	/* 区間 (0, 1) の32ビット浮動小数点数の乱数を生成する */
	public double genrand_real3(){
		return (((double)genrand_int32()) + 0.5) * (1.0 / 4294967296.0);	/* 除数は 2 ^ 32 */ 
	}

	/* 区間 [0, 1) の53ビット浮動小数点数の乱数を生成する */
	public double genrand_res53(){
		int a = (int)(genrand_int32() >>> 5L), b = (int)(genrand_int32() >>> 6L);
		return (a * 67108864.0 + b) * (1.0 / 9007199254740992.0);
	}
	/* 浮動小数点数のバージョンは2002年1月9日に Isaku Wada 氏によって追加されたものである */
}

